--[[
--
-- Module for gathering premade informaton.
--
--   WinningPremadeGroupsEntity = {
--       ["total"] = 5,
--       ["servers"] = {
--           ["ServerA"] = {
--               ["Players"] = {
--                   ["playerA"] = {
--                       -- WinningPlayerEntity
--                       ["Name"] = "",
--                       ["Server"] = "",
--                       ["Class"] = "",
--                       ["Spec"] = "",
--                       ["Label"] = "",
--                   },
--               },
--               ["Healers"] = 0,
--           }
--   }
--
--   WinningPlayerEntity = {
--       Name = "",
--       Server = "",
--       Class = "",
--       Spec = "",
--       Label = "",
--   }
--
--]]

local log = Winning_Log
local C = Winning_Constants
local M = {}
Winning_Premades = M


-- Find premade data for a specific faction.
M.find = function(faction_num)
    local groups = {}

	for i = 1, GetNumBattlefieldScores() do
		local identifier, _, _, _, _, faction, race, _, classToken, _, _, _, _, _, _, talentSpec = GetBattlefieldScore(i)

		if faction == faction_num then
            -- A bug results in some players not having talents.
            -- Just skip them for now.
			if talentSpec then
                local name, server = Winning_Parse_Name(identifier)
                log(format("Found name: %s, server: %s", name, server))

                if groups[server] == nil then
                    groups[server] = {}
                    groups[server]["Players"] = {}
                    groups[server]["Healers"] = 0 
                end

				-- GetBattlefieldScore occasionally returns nil for race, class, and classToken
				-- Assume all three are absent together
				if race then
                    local _class = string.upper(classToken)
                    local class = C.CLASS_SPEC_DISPLAY_LABEL[_class]
                    local label = 'unknown'
                    if class ~= nil then
                        label = class[talentSpec]
                    end

                    local player_entity = {}
                    player_entity['Name'] = name
                    player_entity['Server'] = server
                    player_entity['Class'] = classToken
                    player_entity['Spec'] = talentSpec
                    player_entity['Label'] = label
                    player_entity['IsHealer'] = C.HEALING_SPECS[talentSpec]
                    if player_entity['IsHealer'] then
                        groups[server]['Healers'] = groups[server]['Healers'] + 1
                    end
                    groups[server]['Players'][player_entity['Name']] = player_entity
                else
                    log("Skipping player without race: %s", identifier)
                end
			else
                log("Skipping player without spec: %s", identifier)
			end
        else
            log(string.format("Skipping player with faction: %s", faction))
        end
	end
    local premades = {}
    local total_premades = 0
    for server, group in pairs(groups) do
        if tcount(group['Players']) >= 3 then
            premades[server] = group
            total_premades = total_premades + 1
        else
            log(string.format("Skipping small group from: %s", server))
        end
    end
    local data = {}
    data["servers"] = premades
    data["total"] = total_premades
    return data
end


-- Report faction data for a specific faction.
M.report = function(faction_num)
    log("Reporting seen premades to PARTY chat.")
        
    local premades = M.find(faction_num)

    if premades["total"] > 0 then
        -- Servers
        local premade_msgs = {}
        for server, data in pairs(premades['servers']) do
            Winning_Chat(format("Server: %s", server))
            -- Players
            local count = 0
            for player, details in pairs(data['Players']) do
                local prefix = ' - '
                if details['IsHealer'] then
                    prefix = ' - {skull} '
                end
                local m = format("%s%s: %s", prefix, player, 
                    C.CLASS_SPEC_DISPLAY_LABEL[
                        string.upper(details['Class'])
                    ][details['Spec']])
                Winning_Chat(m)
                --SendChatMessage(m, "PARTY", nil, nil)
                count = count + 1
            end
            if count > 0 then
                local msg = format("{skull} Premade: %s from %s (healers: %s)", count, server, data['Healers'])
                premade_msgs[msg] = true
            end
        end

        for msg, _ in pairs(premade_msgs) do
            SendChatMessage(msg, "PARTY", nil, nil)
        end
    else
        local msg = "{skull} No premades found."
        Winning_Chat(msg)
        SendChatMessage(msg, "PARTY", nil, nil)

    end
end

return M
